#!/bin/sh
#
# ArcaVir 2010 Server installation script.
# Copyright (C) 2009 ArcaBit sp. z o.o.

msg()
{
	echo "install.sh: ${@}"
}

warn()
{
	msg "warning: ${@}"
}

err()
{
	msg "error: ${@}"
}

die()
{
	err "${@}"
	exit 1
}

isprog()
{
        which "$*" > /dev/null 2>&1
}

echo=echo
useradd=useradd
uname=uname
mkdir=mkdir
chown=chown
su=su
id=id
rm=rm
tar=tar
gunzip=gunzip
rsync=rsync
wget=wget
gpg=gpg

PATH=${PATH}:/bin:/usr/bin:/sbin:/usr/sbin

msg 'Welcome to ArcaVir 2010 installation program.'

for prog in \
	"${useradd}" \
	"${uname}" \
	"${mkdir}" \
	"${chown}" \
	"${su}" \
	"${id}" \
	"${rm}" \
	"${tar}" \
	"${gunzip}"; do
	isprog "${prog}" || { err "Program ${prog} not found."; die "Please fix it and run me again."; }
done

[ $(${id} -u) -gt 0 ] && die 'Please run me as root.'

[ -f "base.tar.gz" ] || die 'File base.tar.gz not found. Please fix it and run me again.'

isprog "${rsync}"
if [ $? -gt 0 ]; then
	isprog "${wget}"
	if [ $? -eq 0 ]; then
		warn 'rsync not found, but wget is found. We can continue but update '
		warn 'performance will be strongly degraded. We suggest that you install '
		warn 'rsync after installing ArcaVir.'
	else
		err "Programs ${rsync} and ${wget} not found. Please install rsync "
		err '(wget is not needed then) and run me again. In case you do not '
		die 'want to use rsync, please install wget.'
	fi
fi

isprog "${gpg}"
if [ $? -gt 0 ]; then
	warn 'GnuPG not found. We can continue but update will not '
	warn 'be protected from attacks. We STRONGLY suggest that '
	warn 'you quit now and install GnuPG.'
	warn
	warn 'To continue, please press ENTER.'
	warn 'To quit, please press Ctrl-C.'
	read
fi

msg 'Creating user.'
${useradd} -d /var/lib/arcavir -r -s /bin/false arcabit
[ ${?} -gt 0 ] && die 'Cannot create arcabit user. Please fix it and run me again.'

msg 'Unpacking base components.'
${tar} -C / -zxf base.tar.gz
[ ${?} -gt 0 ] && die 'Cannot untar base.tar.gz. Please fix it and run me again.'

if [ -f "cache.tar.gz" ]; then
	msg 'Unpacking update cache.'
	${mkdir} -p /var/cache/arcavir/update/ || die 'Cache mkdir failed. Please fix it and run me again.'
	${chown} -R arcabit:arcabit /var/cache/arcavir/ || die 'Cache chown failed. Please fix it and run me again.'
	${gunzip} < cache.tar.gz | ${su} -s /bin/sh -c "${tar} -C /var/cache/arcavir/ -xf -" arcabit \
	|| die 'Cannot untar cache. Please fix it an run me again.'

	isprog ${gpg}
	if [ $? -eq 0 ]; then
		msg 'Verifying update cache.'
		${gpg} --no-default-keyring --keyring /var/lib/arcavir/pubring.gpg -q --batch -d < "/var/cache/arcavir/update/files.gpg" > "/var/cache/arcavir/update/files.txt" 2>/dev/null
		if [ $? -gt 0 ]; then
			${rm} -rf "/var/cache/arcavir/update/*"
			die 'GnuPG verify failed! Someone may be tampering with your repository.'
		fi
	fi

	msg 'Propagating.'
	/usr/bin/arcaupdate-propagate-all /var/cache/arcavir/update/ / || die 'Cannot propagate cache. Please fix it and run me again.'
else
	msg 'Downloading ArcaVir 2010 components from the Internet. This will take a while.'
	/usr/bin/arcaupdate || die 'Cannot update. Please fix it and run me again.'
fi

msg 'ArcaVir 2010 installation finished.'
exit 0
